@@ -81,26 +81,37 @@ class LSACMDHelper:
 			print(json.dumps(results, cls = UniversalEncoder, indent=4, sort_keys=True))
 		
 		elif args.grep:
-			print(':'.join(LogonSession.grep_header))
+			if args.directory:
+				print(':'.join(['filename'] + LogonSession.grep_header))
+			else:
+				print(':'.join(LogonSession.grep_header))
 			for result in results:
 				for luid in results[result].logon_sessions:
 					for row in results[result].logon_sessions[luid].to_grep_rows():
+						if args.directory:
+							row = [result] + row
 						print(':'.join(row))
 				for cred in results[result].orphaned_creds:
 					t = cred.to_dict()
 					if t['credtype'] != 'dpapi':
 						if t['password'] is not None:
 							x =  [str(t['credtype']), str(t['domainname']), str(t['username']), '', '', '', '', '', str(t['password'])]
+							if args.directory:
+								x = [result] + x
 							print(':'.join(x))
 					else:
 						t = cred.to_dict()
 						x = [str(t['credtype']), '', '', '', '', '', str(t['masterkey']), str(t['sha1_masterkey']), str(t['key_guid']), '']
+						if args.directory:
+							x = [result] + x
 						print(':'.join(x))
 
 				for pkg, err in results[result].errors:
 					err_str = str(err) +'\r\n' + '\r\n'.join(traceback.format_tb(err.__traceback__))
 					err_str = base64.b64encode(err_str.encode()).decode()
 					x =  [pkg+'_exception_please_report', '', '', '', '', '', '', '', '', err_str]
+					if args.directory:
+						x = [result] + x
 					print(':'.join(x) + '\r\n')
 
 		else:
@@ -168,7 +179,6 @@ class LSACMDHelper:
 				for filename in glob.glob(globdata, recursive=args.recursive):
 					logging.info('Parsing file %s' % filename)
 					try:
-						print('await')
 						mimi = await apypykatz.parse_minidump_file(filename, packages = args.packages)
 						results[filename] = mimi
 					except Exception as e:
