@@ -8,12 +8,12 @@ import os
 import ntpath
 import json
 import hmac
-import hashlib
 import glob
 import sqlite3
 import base64
 import platform
 from hashlib import sha1, pbkdf2_hmac
+
 import xml.etree.ElementTree as ET
 
 from pypykatz import logger
@@ -21,10 +21,9 @@ from pypykatz.dpapi.structures.masterkeyfile import MasterKeyFile
 from pypykatz.dpapi.structures.credentialfile import CredentialFile, CREDENTIAL_BLOB
 from pypykatz.dpapi.structures.blob import DPAPI_BLOB
 from pypykatz.dpapi.structures.vault import VAULT_VCRD, VAULT_VPOL, VAULT_VPOL_KEYS
+from unicrypto.hashlib import md4 as MD4
+from unicrypto.symmetric import AES, MODE_GCM, MODE_CBC
 
-from pypykatz.crypto.unified.aes import AES
-from pypykatz.crypto.unified.aesgcm import AES_GCM
-from pypykatz.crypto.unified.common import SYMMETRIC_MODE
 from pypykatz.commons.common import UniversalEncoder
 
 if platform.system().lower() == 'windows':
@@ -165,9 +164,9 @@ class DPAPI:
 			key1 = None
 		
 		if password or password == '':
-			md4 = hashlib.new('md4')
-			md4.update(password.encode('utf-16le'))
-			nt_hash = md4.digest()
+			ctx = MD4(password.encode('utf-16le'))
+			nt_hash = ctx.digest()
+
 			# Will generate two keys, one with SHA1 and another with MD4
 			key1 = hmac.new(sha1(password.encode('utf-16le')).digest(), (sid + '\0').encode('utf-16le'), sha1).digest()
 		
@@ -484,9 +483,9 @@ class DPAPI:
 		def decrypt_attr(attr, key):
 			if attr.data is not None:
 				if attr.iv is not None:
-					cipher = AES(key, SYMMETRIC_MODE.CBC, iv=attr.iv)
+					cipher = AES(key, MODE_CBC, attr.iv)
 				else:
-					cipher = AES(key, SYMMETRIC_MODE.CBC, iv=b'\x00'*16)
+					cipher = AES(key, MODE_CBC, b'\x00'*16)
 				
 				cleartext = cipher.decrypt(attr.data)
 				return cleartext
@@ -677,12 +676,17 @@ class DPAPI:
 		return results
 		
 	def decrypt_all_chrome_live(self):
+		dbpaths = DPAPI.find_chrome_database_file_live()
+		return self.decrypt_all_chrome(dbpaths)
+		
+		
+	def decrypt_all_chrome(self, dbpaths):
 		results = {}
 		results['logins'] = []
 		results['cookies'] = []
+		results['fmtcookies'] = []
 		localstate_dec = None
 
-		dbpaths = DPAPI.find_chrome_database_file_live()
 		for username in dbpaths:
 			if 'localstate' in dbpaths[username]:
 				with open(dbpaths[username]['localstate'], 'r') as f:
@@ -701,12 +705,14 @@ class DPAPI:
 						nonce = encrypted_value[3:3+12]
 						ciphertext = encrypted_value[3+12:-16]
 						tag = encrypted_value[-16:]
-						cipher = AES_GCM(localstate_dec)
-						dec_val = cipher.decrypt(nonce, ciphertext, tag, auth_data=b'') 
+						cipher = AES(localstate_dec, MODE_GCM)
+						dec_val = cipher.decrypt(nonce, ciphertext, tag, auth_data=b'')
 						results['cookies'].append((dbpaths[username]['cookies'], host_key, name, path, dec_val ))
+						results['fmtcookies'].append(DPAPI.cookieformatter('https://' + host_key, name, path, dec_val))
 					else:
 						dec_val = self.decrypt_blob_bytes(encrypted_value)
 						results['cookies'].append((dbpaths[username]['cookies'], host_key, name, path, dec_val ))
+						results['fmtcookies'].append(DPAPI.cookieformatter('https://' + host_key, name, path, dec_val))
 
 			if 'logindata' in dbpaths[username]:
 				secrets = DPAPI.get_chrome_encrypted_secret(dbpaths[username]['logindata'])
@@ -715,7 +721,7 @@ class DPAPI:
 						nonce = enc_password[3:3+12]
 						ciphertext = enc_password[3+12:-16]
 						tag = enc_password[-16:]
-						cipher = AES_GCM(localstate_dec)
+						cipher = AES(localstate_dec, MODE_GCM)
 						password = cipher.decrypt(nonce, ciphertext, tag, auth_data=b'')
 						results['logins'].append((dbpaths[username]['logindata'], url, user, password))
 					
@@ -723,7 +729,6 @@ class DPAPI:
 						password = self.decrypt_blob_bytes(enc_password)
 						results['logins'].append((dbpaths[username]['logindata'], url, user, password ))
 				
-				
 		return results
 		
 	def get_all_masterkeys_live(self):
@@ -786,13 +791,41 @@ class DPAPI:
 				raise Exception('Failed to obtain SYSTEM privileges! Are you admin? Error: %s' % e)
 			
 			for wificonfig in DPAPI.get_all_wifi_settings_live():
-				if 'enckey' in wificonfig and wificonfig['enckey'] != '':
-					wificonfig['key'] = self.decrypt_securestring_hex(wificonfig['enckey'])
-					yield wificonfig
+				yield self.decrypt_wifi_config_file_inner(wificonfig)
 
 		finally:
 			pm.dropsystem()
 
+	def decrypt_wifi_config_file_inner(self, wificonfig):
+		if 'enckey' in wificonfig and wificonfig['enckey'] != '':
+			wificonfig['key'] = self.decrypt_securestring_hex(wificonfig['enckey'])
+			return wificonfig
+	
+	def decrypt_wifi_config_file(self, configfile):
+		wificonfig = DPAPI.parse_wifi_config_file(configfile)
+		return self.decrypt_wifi_config_file_inner(wificonfig)
+	
+	@staticmethod
+	def cookieformatter(host, name, path, content):
+		"""This is the data format the 'Cookie Quick Manager' uses to load cookies in FireFox"""
+		return {
+			"Host raw": host,      #"https://.pkgs.org/",
+			"Name raw": name,      #"distro_id",
+			"Path raw": path,      #"/",
+			"Content raw": content,   # "196",
+			"Expires": "26-05-2022 21:06:29",       # "12-05-2022 15:59:48",
+			"Expires raw": "1653591989",   # "1652363988",
+			"Send for": "Any type of connection", #"Encrypted connections only",
+			"Send for raw": False,  #"true",
+			"HTTP only raw": False, #"false",
+			"SameSite raw": "lax", #"lax",
+			"This domain only": False, #"Valid for subdomains",
+			"This domain only raw": False, #"false",
+			"Store raw": "firefox-default", #"firefox-default",
+			"First Party Domain": "", #""
+		}
+
+
 # arpparse helper
 def prepare_dpapi_live(methods = [], mkf = None, pkf = None):
 	dpapi = DPAPI()
