@@ -59,89 +59,29 @@ class LsaTemplate_NT6(PackageTemplate):
 			elif WindowsMinBuild.WIN_VISTA.value <= sysinfo.buildnumber < WindowsMinBuild.WIN_7.value:
 				#1
 				template = templates['nt6']['x86']['1']
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#key_pattern.offset_to_DES_key_ptr = -76
-				#key_pattern.offset_to_AES_key_ptr = -21
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 				
 			elif WindowsMinBuild.WIN_7.value <= sysinfo.buildnumber < WindowsMinBuild.WIN_8.value:
 				#2
 				template = templates['nt6']['x86']['2']
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#key_pattern.offset_to_DES_key_ptr = -76
-				#key_pattern.offset_to_AES_key_ptr = -21
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 				
 			elif WindowsMinBuild.WIN_8.value <= sysinfo.buildnumber < WindowsMinBuild.WIN_BLUE.value:
 				#3
 				template = templates['nt6']['x86']['3']
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#key_pattern.offset_to_DES_key_ptr = -69
-				#key_pattern.offset_to_AES_key_ptr = -18
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY8
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 				
 			elif WindowsMinBuild.WIN_BLUE.value <= sysinfo.buildnumber < WindowsMinBuild.WIN_10.value:
 				#4
 				template = templates['nt6']['x86']['4']
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#
-				#key_pattern.offset_to_DES_key_ptr = -69
-				#key_pattern.offset_to_AES_key_ptr = -18
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY81
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 				
 			elif WindowsMinBuild.WIN_10.value <= sysinfo.buildnumber <= WindowsBuild.WIN_10_1507.value:
 				#5
-				template = templates['nt6']['x86']['5']
-
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#key_pattern.offset_to_DES_key_ptr = -79
-				#key_pattern.offset_to_AES_key_ptr = -22
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY81
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
+				template = templates['nt6']['x86']['5']				
 				
-				
-			elif sysinfo.buildnumber > WindowsBuild.WIN_10_1507.value:
+			elif WindowsBuild.WIN_10_1507.value > sysinfo.buildnumber < WindowsBuild.WIN_10_1909.value:
 				#6
 				template = templates['nt6']['x86']['6']
-				#key_pattern = LSADecyptorKeyPattern()
-				#key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
-				#key_pattern.IV_length = 16
-				#key_pattern.offset_to_IV_ptr = 5
-				#key_pattern.offset_to_DES_key_ptr = -79
-				#key_pattern.offset_to_AES_key_ptr = -22
-				#
-				#template.key_pattern = key_pattern
-				#template.key_struct = KIWI_BCRYPT_KEY81
-				#template.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
+			else:
+				#7
+				template = templates['nt6']['x86']['7']
 		
 		elif sysinfo.architecture == KatzSystemArchitecture.X64:
 		
@@ -312,7 +252,29 @@ class KIWI_BCRYPT_KEY81:
 		self.hardkey = KIWI_HARD_KEY(reader)
 		
 	def verify(self):
-		return self.tag == b'KSSM' 
+		return self.tag == b'KSSM'
+
+class KIWI_BCRYPT_KEY81_NEW:
+	def __init__(self, reader):
+		self.size = ULONG(reader).value
+		self.tag  = reader.read(4)	# 'MSSK'
+		self.type = ULONG(reader).value
+		self.unk0 = ULONG(reader).value
+		self.unk1 = ULONG(reader).value
+		self.unk2 = ULONG(reader).value 
+		self.unk3 = ULONG(reader).value
+		self.unk4 = ULONG(reader).value
+		reader.align()
+		self.unk5 = PVOID(reader).value	#before, align in x64
+		self.unk6 = ULONG(reader).value
+		self.unk7 = ULONG(reader).value
+		self.unk8 = ULONG(reader).value
+		self.unk9 = ULONG(reader).value
+		self.unk10 = ULONG(reader).value
+		self.hardkey = KIWI_HARD_KEY(reader)
+		
+	def verify(self):
+		return self.tag == b'KSSM'
 		
 
 class PKIWI_BCRYPT_KEY(POINTER):
@@ -488,6 +450,19 @@ class LSA_x86_6(LsaTemplate_NT6):
 		self.key_struct = KIWI_BCRYPT_KEY81
 		self.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 
+class LSA_x86_7(LsaTemplate_NT6):
+	def __init__(self):
+		LsaTemplate_NT6.__init__(self)
+
+		self.key_pattern = LSADecyptorKeyPattern()
+		self.key_pattern.signature = b'\x6a\x02\x6a\x10\x68'
+		self.key_pattern.IV_length = 16
+		self.key_pattern.offset_to_IV_ptr = 5
+		self.key_pattern.offset_to_DES_key_ptr = -79
+		self.key_pattern.offset_to_AES_key_ptr = -22
+
+		self.key_struct = KIWI_BCRYPT_KEY81_NEW
+		self.key_handle_struct = KIWI_BCRYPT_HANDLE_KEY
 
 
 templates = {
@@ -507,6 +482,7 @@ templates = {
 			'4' : LSA_x86_4(),
 			'5' : LSA_x86_5(),
 			'6' : LSA_x86_6(),
+			'7' : LSA_x86_7(),
 		}
 	}
 }
\ No newline at end of file
