@@ -122,11 +122,11 @@ class pypykatz:
 				print('[-] Failed to parse lsass via handle %s[@%s] Reason: %s' % (pid, lsass_handle, e))
 			
 	@staticmethod
-	def go_live_phandle(lsass_process_handle, packages = ['all']):
+	def go_live_phandle(process_handle, packages = ['all']):
 		if platform.system() != 'Windows':
 			raise Exception('Live parsing will only work on Windows')
 		from pypykatz.commons.readers.local.live_reader import LiveReader
-		reader = LiveReader(lsass_process_handle=lsass_process_handle)
+		reader = LiveReader(process_handle=process_handle)
 		sysinfo = KatzSystemInfo.from_live_reader(reader)
 		mimi = pypykatz(reader.get_buffered_reader(), sysinfo)
 		mimi.start(packages)
@@ -247,7 +247,7 @@ class pypykatz:
 		for lsa_dec_template in LsaTemplate.get_template_brute(self.sysinfo):
 			try:
 				lsa_dec = LsaDecryptor.choose(self.reader, lsa_dec_template, self.sysinfo)
-				logger.debug(lsa_dec.dump())
+				lsa_dec.dump()
 			except:
 				pass
 			else:
@@ -259,7 +259,7 @@ class pypykatz:
 		try:
 			lsa_dec_template = LsaTemplate.get_template(self.sysinfo)
 			lsa_dec = LsaDecryptor.choose(self.reader, lsa_dec_template, self.sysinfo)
-			logger.debug(lsa_dec.dump())
+			lsa_dec.dump()
 		except Exception as e:
 			logger.debug('Failed to automatically detect correct LSA template! Reason: %s' % str(e))
 			lsa_dec = self.get_lsa_bruteforce()
@@ -383,13 +383,7 @@ class pypykatz:
 				self.get_ssp()
 			except Exception as e:
 				self.errors.append(('ssp', e))
-
-		if 'livessp' in packages or 'all' in packages:
-			try:
-				self.get_livessp()
-			except Exception as e:
-				self.errors.append(('livessp', e))
-
+		
 		if 'dpapi' in packages or 'all' in packages:
 			try:
 				self.get_dpapi()
@@ -402,3 +396,11 @@ class pypykatz:
 			except Exception as e:
 				self.errors.append(('cloudap', e))
 
+		if 'livessp' in packages or 'all' in packages:
+			try:
+				self.get_livessp()
+			except Exception as e:
+				self.errors.append(('livessp', e))
+
+		
+
